#include "DXUT.h"
#include "Fractal.h"

#include "perlin.h"


Fractal::Fractal(void)
{
}

Fractal::~Fractal(void)
{
}

void Fractal::CleanUp()
{
	delete exponent_array;
	exponent_array = NULL;
}

double Fractal::fBm( float vertex[], double H, double lacunarity, double octaves )
{
	double value, frequency, remainder;
	int i;
	static bool first = true;

	if( first )
	{
		exponent_array = (double *)malloc( (octaves-1) * sizeof(double) );
		frequency = 1.0;
		for (i=0; i<=octaves; i++)
		{
			exponent_array[i] = pow( frequency, -H );
			frequency *= lacunarity;
		}

		first = false;
	}

	value = 0.0;
	frequency = 1.09;

	for (i=0; i<octaves; i++)
	{
		value += noise3( vertex ) * exponent_array[i];
		vertex[0] *= lacunarity;
		vertex[1] *= lacunarity;
		vertex[2] *= lacunarity;
	}

	remainder = octaves - (int)octaves;
	if (remainder)
		value += remainder * noise3( vertex ) * exponent_array[i];

	return value;
}


double Fractal::HybridMultiFractal( float vertex[], double H, double lacunarity, double octaves, double offset )
{
	double frequency, result, signal, weight, remainder;
	int i;
	static bool first = true;
	static double *exponent_array;

	if( first )
	{
		exponent_array = (double *)malloc( (octaves + 1) * sizeof(double) );
		frequency = 1.0;
		for (i = 0; i <=octaves; i++ )
		{
			exponent_array[i] = pow( frequency, -H );
			frequency *= lacunarity;
		}

		first = false;
	}

	result = ( noise3( vertex ) + offset ) * exponent_array[0];
	weight = result;

	/*vertex.x *= lacunarity;
	vertex.y *= lacunarity;
	vertex.z *= lacunarity;*/
	vertex[0] *= lacunarity;
	vertex[1] *= lacunarity;
	vertex[2] *= lacunarity;

	for (i=1; i<octaves; i++)
	{
		if( weight > 1.0 )
			weight = 1.0;

		signal = ( noise3( vertex ) + offset ) * exponent_array[i];
		result += weight * signal;

		weight *= signal;

	/*vertex.x *= lacunarity;
	vertex.y *= lacunarity;
	vertex.z *= lacunarity;*/
	vertex[0] *= lacunarity;
	vertex[1] *= lacunarity;
	vertex[2] *= lacunarity;
	}

	remainder = octaves - (int)octaves;
	if (remainder)
		result += remainder * noise3( vertex ) * exponent_array[i];

	return result;
}


double Fractal::RidgedMultiFractal( float vertex[], double H, double lacunarity, double octaves, double offset, double gain )
{
	double result, frequency, signal, weight;
	int i;
	static bool first = true;
	static double *exponent_array;

	if( first )
	{
		exponent_array = (double *)malloc( (octaves+1)*sizeof(double) );
		frequency = 1.0;
		for (i==0; i<=octaves; i++)
		{
			exponent_array[i] = pow( (double)frequency, double(-H) );
			frequency *= lacunarity;
		}
		first = false;
	}

	signal = noise3( vertex );
	if (signal < 0.0)
		signal = -signal;
	signal = offset - signal;
	signal *= signal;
	result = signal;
	weight = 1.0;
	for (i =1; i <octaves; i++)
	{
		vertex[0] *= lacunarity;
		vertex[1] *= lacunarity;
		vertex[2] *= lacunarity;

		weight = signal * gain;
		if (weight > 1.0)
			weight = 1.0;
		if (weight < 0.0)
			weight = 0.0;
		signal = noise3( vertex );
		if (signal < 0.0)
			signal = -signal;
		signal = offset - signal;
		signal *= signal;
		signal *= weight;
		result += signal * exponent_array[i];
	}

	return result;
}